@@ -2,6 +2,7 @@ import datetime
 import math
 import re
 import pymongo
+from pandas import DataFrame
 from util_hj3415 import utils
 from .setting import load as load_setting
 from collections import OrderedDict
@@ -15,15 +16,32 @@ logger.addHandler(ch)
 logger.setLevel(logging.WARNING)
 
 
+def chk_deactivate_mongo(func):
+    def wrapper(*args, **kwargs):
+        addr = load_setting()['mongo']
+        if addr == '' or addr is None:
+            print('mongo db is deactivated..')
+            return None
+        else:
+            return func(*args, **kwargs)
+    return wrapper
+
+
 class Base:
     def __init__(self, db, col):
         self.mongo_addr = load_setting()['mongo']
 
         # mongodb 에 연결
-        self.client = pymongo.MongoClient(self.mongo_addr)
-        self.db = db
-        self.col = col
-        logger.info(f"__init__ set db : {self.db}\tcol : {self.col}")
+        if self.mongo_addr != '':
+            self.client = pymongo.MongoClient(self.mongo_addr)
+            self.db = db
+            self.col = col
+            logger.info(f"__init__ set db : {self.db}\tcol : {self.col}")
+        else:
+            print('mongo db is deactivated..')
+            self.client = None
+            self.db = None
+            self.col = None
 
     # ======================Base 만 가지는 함수들 ===========================
 
@@ -49,11 +67,13 @@ class Base:
     # get 관련함수는 self 를 사용하나 drop 관련 함수를 신중을 위해 self 를 사용하지 않고 반드시 인자로 받는다.
 
     @staticmethod
+    @chk_deactivate_mongo
     def get_all_db() -> list:
         client = pymongo.MongoClient(load_setting()['mongo'])
         return sorted(client.list_database_names())
 
     @staticmethod
+    @chk_deactivate_mongo
     def drop_all_db():
         client = pymongo.MongoClient(load_setting()['mongo'])
         pre_db = Base.get_all_db()
@@ -63,6 +83,7 @@ class Base:
         print(f"Drop all db..{len(pre_db)} -> {len(post_db)}")
 
     @staticmethod
+    @chk_deactivate_mongo
     def drop_db(db: str):
         client = pymongo.MongoClient(load_setting()['mongo'])
         pre_db = Base.get_all_db()
@@ -205,6 +226,8 @@ class Corps(Base):
 
     def __init__(self, code: str, page='c101'):
         if utils.is_6digit(code) and page in self.COL_TITLE:
+            self.code = code
+            self.page = page
             super().__init__(db=code, col=page)
         else:
             raise ValueError(f'Invalid value : {code}(6 digit) / {page}({self.COL_TITLE})')
@@ -215,6 +238,7 @@ class Corps(Base):
         # code 가 6자리 숫자인지 확인 후 세팅
         if utils.is_6digit(code):
             logger.info(f'Set db : {self.db} -> {code}')
+            self.code = code
             self.db = code
         else:
             raise ValueError(f'Invalid value : {code}(6 digit)')
@@ -224,6 +248,7 @@ class Corps(Base):
         if page in self.COL_TITLE:
             logger.info(f'Set col : {self.col} -> {page}')
             self.col = page
+            self.page = page
         else:
             raise ValueError(f'Invalid col name : {page}({self.COL_TITLE})')
 
@@ -547,7 +572,10 @@ class C1034(Corps):
         타이틀에 해당하는 년도, 분기 딕셔너리를 반환한다.
         *** 중복되는 title은 취급하지 않기로함...get_all_title()함수에서 정리함.
         """
-        return self.get_all()[title]
+        try:
+            return self.get_all()[title]
+        except KeyError:
+            return {}
 
     def sum_recent_4q(self, title: str) -> tuple:
         """
@@ -583,8 +611,16 @@ class C1034(Corps):
         # reverse = False 이면 오래된것부터 최근순으로 정렬한다.
         od = OrderedDict(sorted(self.find(title=title).items(), reverse=False))
         logger.info(f'{title} : {od}')
-        last_one = od.popitem(last=True)
-        if math.isnan(last_one[1]):
+        try:
+            last_one = od.popitem(last=True)
+        except KeyError:
+            # when dictionary is empty
+            return None, 0
+        logger.info(f'last_one : {last_one}')
+        if isinstance(last_one[1], str):
+            # last_one : ('Unnamed: 1', '데이터가 없습니다.') 인 경우
+            return None, 0
+        elif math.isnan(last_one[1]):
             last_one = od.popitem(last=True)
             if last_one is None:
                 return None, 0
@@ -651,3 +687,37 @@ class C103(C1034):
         self.client[self.db][self.col].delete_many({})
         result = self.client[self.db][self.col].insert_many(c103_list)
         return result.acknowledged
+
+
+class Noti(Base):
+    def __init__(self):
+        super().__init__(db='noti', col='noti')
+
+    def save(self, noti_dict: dict) -> bool:
+        """
+        noti_dict 구조
+        {'code': '005930',
+        'rcept_no': '20210514000624',
+        'rcept_dt': '20210514',
+        'report_nm': '임원ㆍ주요주주특정증권등소유상황보고서',
+        'point': 2,
+        'text': '등기임원이 1.0억 이상 구매하지 않음.'}
+        """
+        self.client[self.db][self.col].delete_many({'rcept_no': {"$eq": noti_dict['rcept_no']}})
+        result = self.client[self.db][self.col].insert_one(noti_dict)
+        return result.acknowledged
+
+    def get_data(self) -> DataFrame:
+        try:
+            df = DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+        except KeyError:
+            df = DataFrame()
+        return df
+
+    def cleaning_data(self, days_ago=15):
+        border_date_str = (datetime.datetime.today() - datetime.timedelta(days=days_ago)).strftime('%Y%m%d')
+        try:
+            self.client[self.db][self.col].delete_many({'rcept_dt': {'$lt': border_date_str}})
+            logger.info(f'Delete noti data before {days_ago} days ago..')
+        except:
+            logger.error(f'Error occurred while delete noti data..')
\ No newline at end of file
