@@ -2,11 +2,12 @@
 from sqlalchemy import text, Column, String, Integer, create_engine, Float, TIMESTAMP, MetaData
 from sqlalchemy.sql.expression import func
 from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import sessionmaker
 import os
 from .setting import load as load_setting
 from util_hj3415 import utils
-from pandas import DataFrame
+import pandas as pd
 
 import logging
 logger = logging.getLogger(__name__)
@@ -22,9 +23,10 @@ logger.setLevel(logging.WARNING)
 
 BaseCorps = declarative_base()
 BaseMI = declarative_base()
+BaseNoti = declarative_base()
 
 
-class C101_stucture(BaseCorps):
+class C101Structure(BaseCorps):
     __tablename__ = 'c101'
     date = Column("date", String, primary_key=True, nullable=False)
     코드 = Column("코드", String)
@@ -51,13 +53,16 @@ class C101_stucture(BaseCorps):
         return '/'.join([self.date, self.코드, str(self.종목명), str(self.업종), str(self.주가)])
 
 
-class Stamp_stucture(BaseCorps):
+class StampStructure(BaseCorps):
     __tablename__ = 'stamp'
     id = Column(Integer, nullable=False, primary_key=True, autoincrement=True)
     name = Column(String)
     sqltime = Column(TIMESTAMP(timezone=True), server_default=func.now(), onupdate=func.current_timestamp())
 
 
+# ===========================================================
+
+
 class Kospi(BaseMI):
     __tablename__ = 'kospi'
     date = Column("date", String, primary_key=True, nullable=False)
@@ -178,11 +183,41 @@ class Usdidx(BaseMI):
 # ===========================================================
 
 
+class NotiStructure(BaseNoti):
+    __tablename__ = 'noti'
+    rcept_no = Column('rcept_no', String, nullable=False, primary_key=True)
+    code = Column('code', String)
+    rcept_dt = Column('rcept_dt', String)
+    report_nm = Column('report_nm', String)
+    point = Column("point", Integer)
+    text = Column('text', String)
+
+
+# ===========================================================
+
+def chk_deactivate_sqlite3(func):
+    def wrapper(*args, **kwargs):
+        path = load_setting()['sqlite3']
+        if path == '' or path is None:
+            print('sqlite3 db is deactivated..')
+            return None
+        else:
+            return func(*args, **kwargs)
+    return wrapper
+
+
 class Base:
     def __init__(self, folder_name: str, db_name: str):
-        self.engine = self.make_engine(folder_name, db_name)
+        self.sqlite3_path = load_setting()['sqlite3']
+
+        if self.sqlite3_path != '':
+            self.engine = self.make_engine(folder_name, db_name)
+        else:
+            print('sqlite3 db is deactivated..')
+            self.engine = None
 
     @staticmethod
+    @chk_deactivate_sqlite3
     def make_engine(folder_name: str, db_name: str):
         # make folder - /db/{folder_name}
         path = os.path.join(load_setting()['sqlite3'], folder_name)
@@ -192,11 +227,20 @@ class Base:
         logger.info(f"Connect to {dsn}")
         return create_engine(dsn, echo=False)
 
+    @chk_deactivate_sqlite3
     def get_tables(self) -> tuple:
         meta = MetaData()
         meta.reflect(bind=self.engine)
         return tuple(meta.tables.keys())
 
+    def _get_session(self):
+        if self.engine is None:
+            return None
+        else:
+            BaseCorps.metadata.create_all(self.engine)
+            Session = sessionmaker(bind=self.engine)
+            return Session()
+
 
 class Corps(Base):
     # mongodb와 다른점은 stamp 테이블과 c106이 c106y,c106q로 각각의 테이블을 가진다.
@@ -204,14 +248,12 @@ class Corps(Base):
              'c103손익계산서q', 'c103재무상태표q', 'c103현금흐름표q',
              'c103손익계산서y', 'c103재무상태표y', 'c103현금흐름표y', 'dart', 'stamp')
 
-    def __init__(self, code: str, table: str):
+    def __init__(self, code: str, table: str = 'c101'):
         if utils.is_6digit(code) and table in self.TABLE:
             self.code = code
             self.table = table
             super().__init__(folder_name='corps', db_name=self.code)
-            BaseCorps.metadata.create_all(self.engine)
-            Session = sessionmaker(bind=self.engine)
-            self.session = Session()
+            self.session = self._get_session()
         else:
             raise ValueError(f'Invalid value : {code}(6 digit) / {table}({self.TABLE})')
 
@@ -222,10 +264,7 @@ class Corps(Base):
         if utils.is_6digit(code):
             logger.info(f'Set db : {self.code} -> {code}')
             self.code = code
-            self.engine = self.make_engine(folder_name='corps', db_name=self.code)
-            BaseCorps.metadata.create_all(self.engine)
-            Session = sessionmaker(bind=self.engine)
-            self.session = Session()
+            self.session = self._get_session()
         else:
             raise ValueError(f'Invalid value : {code}(6 digit)')
 
@@ -245,7 +284,7 @@ class Corps(Base):
             raise ValueError(f'Invalid table : {table} / {self.TABLE}')
 
     def stamping(self):
-        stamp = Stamp_stucture(name=self.table)
+        stamp = StampStructure(name=self.table)
         self.session.add(stamp)
         logger.info(f"Stamping.. name:'{self.table}'")
         self.session.commit()
@@ -264,29 +303,28 @@ class C101(Corps):
         if c101_dict['코드'] != self.code:
             raise ValueError("Code isn't equal input data and db data..")
         if sorted(c101_struc) == sorted(c101_dict.keys()):
-            self.session.query(C101_stucture).filter(C101_stucture.date >= c101_dict['date']).delete()
-            c101 = C101_stucture(date=c101_dict['date'],
-                                 코드=c101_dict['코드'],
-                                 종목명=c101_dict['종목명'],
-                                 업종=c101_dict['업종'],
-                                 주가=c101_dict['주가'],
-                                 거래량=c101_dict['거래량'],
-                                 EPS=c101_dict['EPS'],
-                                 BPS=c101_dict['BPS'],
-                                 PER=c101_dict['PER'],
-                                 업종PER=c101_dict['업종PER'],
-                                 PBR=c101_dict['PBR'],
-                                 배당수익률=c101_dict['배당수익률'],
-                                 최고52주=c101_dict['최고52주'],
-                                 최저52주=c101_dict['최저52주'],
-                                 거래대금=c101_dict['거래대금'],
-                                 시가총액=c101_dict['시가총액'],
-                                 베타52주=c101_dict['베타52주'],
-                                 발행주식=c101_dict['발행주식'],
-                                 유통비율=c101_dict['유통비율'],
-                                 intro=c101_dict['intro'])
-            self.session.add(c101)
-            logger.info(self.session.query(C101_stucture).filter_by(date=c101_dict['date']).first())
+            self.session.query(C101Structure).filter(C101Structure.date >= c101_dict['date']).delete()
+            self.session.add(C101Structure(date=c101_dict['date'],
+                                           코드=c101_dict['코드'],
+                                           종목명=c101_dict['종목명'],
+                                           업종=c101_dict['업종'],
+                                           주가=c101_dict['주가'],
+                                           거래량=c101_dict['거래량'],
+                                           EPS=c101_dict['EPS'],
+                                           BPS=c101_dict['BPS'],
+                                           PER=c101_dict['PER'],
+                                           업종PER=c101_dict['업종PER'],
+                                           PBR=c101_dict['PBR'],
+                                           배당수익률=c101_dict['배당수익률'],
+                                           최고52주=c101_dict['최고52주'],
+                                           최저52주=c101_dict['최저52주'],
+                                           거래대금=c101_dict['거래대금'],
+                                           시가총액=c101_dict['시가총액'],
+                                           베타52주=c101_dict['베타52주'],
+                                           발행주식=c101_dict['발행주식'],
+                                           유통비율=c101_dict['유통비율'],
+                                           intro=c101_dict['intro']))
+            logger.info(self.session.query(C101Structure).filter_by(date=c101_dict['date']).first())
             self.session.commit()
             return True
         else:
@@ -307,7 +345,7 @@ class C103(Corps):
         else:
             raise ValueError(f'Invalid value : {page}')
 
-    def save(self, c103_df: DataFrame) -> bool:
+    def save(self, c103_df: pd.DataFrame) -> bool:
         """
         데이터프레임 형식의 인자를 받아 저장한다.
         """
@@ -322,7 +360,7 @@ class C104(Corps):
     def __init__(self, code: str, page: str):
         super().__init__(code=code, table=page)
 
-    def save(self, c104_df: DataFrame) -> bool:
+    def save(self, c104_df: pd.DataFrame) -> bool:
         """
         데이터프레임 형식의 인자를 받아 저장한다.
         """
@@ -343,7 +381,7 @@ class C106(Corps):
         else:
             raise ValueError(f'Invalid page : {page}(c106q or c106y)')
 
-    def save(self, c106_df: DataFrame) -> bool:
+    def save(self, c106_df: pd.DataFrame) -> bool:
         """
         데이터프레임 형식의 인자를 받아 저장한다.
         """
@@ -358,7 +396,7 @@ class C108(Corps):
     def __init__(self, code: str):
         super().__init__(code=code, table='c108')
 
-    def save(self, c108_df: DataFrame) -> bool:
+    def save(self, c108_df: pd.DataFrame) -> bool:
         """
         데이터프레임 형식의 인자를 받아 저장한다.
         """
@@ -418,3 +456,43 @@ class MI(Base):
             logger.info(f'Add {self.table} : {mi_dict}')
         self.session.commit()
         return True
+
+
+class Noti(Base):
+    def __init__(self):
+        self.table = NotiStructure.__tablename__
+        super().__init__(folder_name='noti', db_name='noti')
+        BaseNoti.metadata.create_all(self.engine)
+        Session = sessionmaker(bind=self.engine)
+        self.session = Session()
+
+    def save(self, noti_dict: dict) -> bool:
+        """
+        noti_dict 구조
+        {'code': '005930',
+        'rcept_no': '20210514000624',
+        'rcept_dt': '20210514',
+        'report_nm': '임원ㆍ주요주주특정증권등소유상황보고서',
+        'point': 2,
+        'text': '등기임원이 1.0억 이상 구매하지 않음.'}
+        """
+        self.session.add(NotiStructure(rcept_no=noti_dict['rcept_no'],
+                                       code=noti_dict['code'],
+                                       rcept_dt=noti_dict['rcept_dt'],
+                                       report_nm=noti_dict['report_nm'],
+                                       point=noti_dict['point'],
+                                       text=noti_dict['text']))
+        try:
+            self.session.commit()
+            logger.info(f'Add {self.table} : {noti_dict}')
+        except IntegrityError:
+            # 같은 내용 중복저상시 경고 로그 표시하고 저장하지 않음.
+            logger.warning(f"rcept_no : {noti_dict['rcept_no']}  exists already in db.")
+        return True
+
+    def get_data(self) -> pd.DataFrame:
+        df = pd.read_sql_query(text(f"SELECT * FROM {self.table}"), con=self.engine)
+        return df
+
+    def cleaning_data(self, days_ago=15):
+        logger.warning("Deleting sqlite noti data is not implement yet..")
\ No newline at end of file
